@@ -4,7 +4,7 @@
    way... */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2005 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -35,7 +35,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: inet.c,v 1.8.2.5 2004/06/10 17:59:18 dhankins Exp $ Copyright (c) 2004 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: inet.c,v 1.8.2.7 2005/03/03 16:55:22 dhankins Exp $ Copyright (c) 2004-2005 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -49,6 +49,12 @@ struct iaddr subnet_number (addr, mask)
 	int i;
 	struct iaddr rv;
 
+	if (addr.len > sizeof(addr.iabuf))
+		log_fatal("subnet_number():%s:%d: Invalid addr length.", MDL);
+	if (addr.len != mask.len)
+		log_fatal("subnet_number():%s:%d: Addr/mask length mismatch.",
+			  MDL);
+
 	rv.len = 0;
 
 	/* Both addresses must have the same length... */
@@ -75,6 +81,12 @@ struct iaddr ip_addr (subnet, mask, host_address)
 	struct iaddr rv;
 	unsigned char habuf [sizeof swaddr];
 
+	if (subnet.len > sizeof(subnet.iabuf))
+		log_fatal("ip_addr():%s:%d: Invalid addr length.", MDL);
+	if (subnet.len != mask.len)
+		log_fatal("ip_addr():%s:%d: Addr/mask length mismatch.",
+			  MDL);
+
 	swaddr = htonl (host_address);
 	memcpy (habuf, &swaddr, sizeof swaddr);
 
@@ -115,6 +127,12 @@ struct iaddr broadcast_addr (subnet, mask)
 	int i, j, k;
 	struct iaddr rv;
 
+	if (subnet.len > sizeof(subnet.iabuf))
+		log_fatal("broadcast_addr():%s:%d: Invalid addr length.", MDL);
+	if (subnet.len != mask.len)
+		log_fatal("broadcast_addr():%s:%d: Addr/mask length mismatch.",
+			  MDL);
+
 	if (subnet.len != mask.len) {
 		rv.len = 0;
 		return rv;
@@ -136,6 +154,12 @@ u_int32_t host_addr (addr, mask)
 	u_int32_t swaddr;
 	struct iaddr rv;
 
+	if (addr.len > sizeof(addr.iabuf))
+		log_fatal("host_addr():%s:%d: Invalid addr length.", MDL);
+	if (addr.len != mask.len)
+		log_fatal("host_addr():%s:%d: Addr/mask length mismatch.",
+			  MDL);
+
 	rv.len = 0;
 
 	/* Mask out the network bits... */
@@ -153,6 +177,9 @@ u_int32_t host_addr (addr, mask)
 int addr_eq (addr1, addr2)
 	struct iaddr addr1, addr2;
 {
+	if (addr1.len > sizeof(addr1.iabuf))
+		log_fatal("addr_eq():%s:%d: Invalid addr length.", MDL);
+
 	if (addr1.len != addr2.len)
 		return 0;
 	return memcmp (addr1.iabuf, addr2.iabuf, addr1.len) == 0;
@@ -165,22 +192,8 @@ char *piaddr (addr)
 	char *s = pbuf;
 	int i;
 
-	if (addr.len == 0) {
-		strcpy (s, "<null address>");
-	}
-	for (i = 0; i < addr.len; i++) {
-		sprintf (s, "%s%d", i ? "." : "", addr.iabuf [i]);
-		s += strlen (s);
-	}
-	return pbuf;
-}
-
-char *piaddr1 (addr)
-	struct iaddr addr;
-{
-	static char pbuf [4 * 16];
-	char *s = pbuf;
-	int i;
+	if (addr.len > sizeof(addr.iabuf))
+		log_fatal("piaddr():%s:%d: Address length too long.", MDL);
 
 	if (addr.len == 0) {
 		strcpy (s, "<null address>");
@@ -195,39 +208,40 @@ char *piaddr1 (addr)
 char *piaddrmask (struct iaddr addr, struct iaddr mask,
 		  const char *file, int line)
 {
-	char *s, *t;
-	int i, mw;
-	unsigned len;
+	char *s, tbuf[sizeof("255.255.255.255/32")];
+	int mw;
+	unsigned i, oct, bit;
 
-	for (i = 0; i < 32; i++) {
-		if (!mask.iabuf [3 - i / 8])
-			i += 7;
-		else if (mask.iabuf [3 - i / 8] & (1 << (i % 8)))
+	if (addr.len != 4)
+		log_fatal("piaddrmask():%s:%d: Address length %d invalid",
+			  MDL, addr.len);
+	if (addr.len != mask.len)
+		log_fatal("piaddrmask():%s:%d: Address and mask size mismatch",
+			  MDL);
+
+	/* Determine netmask width in bits. */
+	for (mw = 32; mw > 0; ) {
+		oct = (mw - 1) / 8;
+		bit = 0x80 >> ((mw - 1) % 8);
+		if (!mask.iabuf [oct])
+			mw -= 8;
+		else if (mask.iabuf [oct] & bit)
 			break;
-	}
-	mw = 32 - i;
-	len = mw > 9 ? 2 : 1;
-	len += 4;	/* three dots and a slash. */
-	for (i = 0; i < (mw / 8) + 1; i++) {
-		if (addr.iabuf [i] > 99)
-			len += 3;
-		else if (addr.iabuf [i] > 9)
-			len += 2;
 		else
-			len++;
+			mw--;
 	}
-	s = dmalloc (len + 1, file, line);
+
+	s = tbuf;
+	for (i = 0 ; i <= oct ; i++) {
+		sprintf(s, "%s%d", i ? "." : "", addr.iabuf[i]);
+		s += strlen(s);
+	}
+	sprintf(s, "/%d", mw);
+
+	s = dmalloc (strlen(tbuf) + 1, file, line);
 	if (!s)
 		return s;
-	t = s;
-	sprintf (t, "%d", addr.iabuf [0]);
-	t += strlen (t);
-	for (i = 1; i < (mw / 8) + 1; i++) {
-		sprintf (t, ".%d", addr.iabuf [i]);
-		t += strlen (t);
-	}
-	*t++ = '/';
-	sprintf (t, "%d", mw);
+	strcpy(s, tbuf);
 	return s;
 }
 
