@@ -3,7 +3,7 @@
    DHCP options parsing and reassembly. */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: options.c,v 1.85.2.13 2004/06/10 17:59:19 dhankins Exp $ Copyright (c) 2004 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: options.c,v 1.85.2.34 2006/02/22 22:43:27 dhankins Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #define DHCP_OPTION_DATA
@@ -465,8 +465,10 @@ int cons_options (inpacket, outpacket, lease, client_state,
 	unsigned priority_list [PRIORITY_COUNT];
 	int priority_len;
 	unsigned char buffer [4096];	/* Really big buffer... */
-	unsigned main_buffer_size;
+	unsigned main_buffer_size, mb_max;
 	unsigned mainbufix, bufix, agentix;
+	int fileix;
+	int snameix;
 	unsigned option_size;
 	unsigned length;
 	int i;
@@ -475,6 +477,8 @@ int cons_options (inpacket, outpacket, lease, client_state,
 	pair pp, *hash;
 	int need_endopt = 0;
 	int have_sso = 0;
+	int ocount = 0;
+	int ofbuf1=0, ofbuf2=0;
 
 	memset (&ds, 0, sizeof ds);
 
@@ -521,8 +525,10 @@ int cons_options (inpacket, outpacket, lease, client_state,
 		main_buffer_size = 576 - DHCP_FIXED_LEN;
 
 	/* Set a hard limit at the size of the output buffer. */
-	if (main_buffer_size > sizeof buffer)
-		main_buffer_size = sizeof buffer;
+	mb_max = sizeof(buffer) - (((overload & 1) ? DHCP_FILE_LEN : 0) +
+				   ((overload & 2) ? DHCP_SNAME_LEN : 0));
+	if (main_buffer_size > mb_max)
+		main_buffer_size = mb_max;
 
 	/* Preload the option priority list with mandatory options. */
 	priority_len = 0;
@@ -626,18 +632,35 @@ int cons_options (inpacket, outpacket, lease, client_state,
 				DHO_VENDOR_ENCAPSULATED_OPTIONS;
 	}
 
+	/* Figure out the overload buffer offset(s). */
+	if (overload) {
+		ofbuf1 = main_buffer_size - 4;
+		if (overload == 3)
+			ofbuf2 = main_buffer_size - 4 + DHCP_FILE_LEN;
+	}
+
 	/* Copy the options into the big buffer... */
-	option_size = store_options (buffer,
-				     (main_buffer_size - 7 +
+	option_size = store_options (&ocount, buffer,
+				     (main_buffer_size - 4 +
 				      ((overload & 1) ? DHCP_FILE_LEN : 0) +
 				      ((overload & 2) ? DHCP_SNAME_LEN : 0)),
 				     inpacket, lease, client_state,
 				     in_options, cfg_options, scope,
 				     priority_list, priority_len,
-				     main_buffer_size,
-				     (main_buffer_size +
-				      ((overload & 1) ? DHCP_FILE_LEN : 0)),
-				     terminate, vuname);
+				     ofbuf1, ofbuf2, terminate, vuname);
+	/* If store_options failed. */
+	if (option_size == 0)
+		return 0;
+	if (overload) {
+		if (ocount == 1 && (overload & 1))
+			overload = 1;
+		else if (ocount == 1 && (overload & 2))
+			overload = 2;
+		else if (ocount == 3)
+			overload = 3;
+		else
+			overload = 0;
+	}
 
 	/* Put the cookie up front... */
 	memcpy (outpacket -> options, DHCP_OPTIONS_COOKIE, 4);
@@ -647,73 +670,46 @@ int cons_options (inpacket, outpacket, lease, client_state,
 	   option at the beginning.  If we can, though, just store the
 	   whole thing in the packet's option buffer and leave it at
 	   that. */
-	if (option_size <= main_buffer_size - mainbufix) {
-		memcpy (&outpacket -> options [mainbufix],
-			buffer, option_size);
-		mainbufix += option_size;
-		agentix = mainbufix;
-		if (mainbufix < main_buffer_size)
-			need_endopt = 1;
-		length = DHCP_FIXED_NON_UDP + mainbufix;
-	} else {
+	memcpy (&outpacket -> options [mainbufix],
+		buffer, option_size);
+	mainbufix += option_size;
+	if (overload) {
 		outpacket -> options [mainbufix++] = DHO_DHCP_OPTION_OVERLOAD;
 		outpacket -> options [mainbufix++] = 1;
-		if (option_size > main_buffer_size - mainbufix + DHCP_FILE_LEN)
-			outpacket -> options [mainbufix++] = 3;
-		else
-			outpacket -> options [mainbufix++] = 1;
+		outpacket -> options [mainbufix++] = overload;
 
-		memcpy (&outpacket -> options [mainbufix],
-			buffer, main_buffer_size - mainbufix);
-		length = DHCP_FIXED_NON_UDP + main_buffer_size;
-		agentix = main_buffer_size;
-
-		bufix = main_buffer_size - mainbufix;
 		if (overload & 1) {
-			if (option_size - bufix <= DHCP_FILE_LEN) {
-				memcpy (outpacket -> file,
-					&buffer [bufix], option_size - bufix);
-				mainbufix = option_size - bufix;
-				if (mainbufix < DHCP_FILE_LEN)
-					outpacket -> file [mainbufix++]
-						= DHO_END;
-				while (mainbufix < DHCP_FILE_LEN)
-					outpacket -> file [mainbufix++]
-						= DHO_PAD;
+			memcpy (outpacket -> file,
+				&buffer [ofbuf1], DHCP_FILE_LEN);
+		}
+		if (overload & 2) {
+			if (ofbuf2) {
+				memcpy (outpacket -> sname, &buffer [ofbuf2],
+					DHCP_SNAME_LEN);
 			} else {
-				memcpy (outpacket -> file,
-					&buffer [bufix], DHCP_FILE_LEN);
-				bufix += DHCP_FILE_LEN;
+				memcpy (outpacket -> sname, &buffer [ofbuf1],
+					DHCP_SNAME_LEN);
 			}
 		}
-		if ((overload & 2) && option_size < bufix) {
-			memcpy (outpacket -> sname,
-				&buffer [bufix], option_size - bufix);
-
-			mainbufix = option_size - bufix;
-			if (mainbufix < DHCP_SNAME_LEN)
-				outpacket -> file [mainbufix++]
-					= DHO_END;
-			while (mainbufix < DHCP_SNAME_LEN)
-				outpacket -> file [mainbufix++]
-					= DHO_PAD;
-		}
 	}
+	agentix = mainbufix;
+	if (mainbufix < main_buffer_size)
+		need_endopt = 1;
+	length = DHCP_FIXED_NON_UDP + mainbufix;
 
 	/* Now hack in the agent options if there are any. */
 	priority_list [0] = DHO_DHCP_AGENT_OPTIONS;
 	priority_len = 1;
 	agentix +=
-		store_options (&outpacket -> options [agentix],
-			       1500 - DHCP_FIXED_LEN - agentix,
+		store_options (0, &outpacket -> options [agentix],
+			       DHCP_OPTION_LEN - agentix,
 			       inpacket, lease, client_state,
 			       in_options, cfg_options, scope,
 			       priority_list, priority_len,
-			       1500 - DHCP_FIXED_LEN - agentix,
-			       1500 - DHCP_FIXED_LEN - agentix, 0, (char *)0);
+			       0, 0, 0, (char *)0);
 
 	/* Tack a DHO_END option onto the packet if we need to. */
-	if (agentix < 1500 - DHCP_FIXED_LEN && need_endopt)
+	if (agentix < DHCP_OPTION_LEN && need_endopt)
 		outpacket -> options [agentix++] = DHO_END;
 
 	/* Figure out the length. */
@@ -723,9 +719,10 @@ int cons_options (inpacket, outpacket, lease, client_state,
 
 /* Store all the requested options into the requested buffer. */
 
-int store_options (buffer, buflen, packet, lease, client_state,
+int store_options (ocount, buffer, buflen, packet, lease, client_state,
 		   in_options, cfg_options, scope, priority_list, priority_len,
 		   first_cutoff, second_cutoff, terminate, vuname)
+	int *ocount;
 	unsigned char *buffer;
 	unsigned buflen;
 	struct packet *packet;
@@ -740,14 +737,30 @@ int store_options (buffer, buflen, packet, lease, client_state,
 	int terminate;
 	const char *vuname;
 {
-	int bufix = 0;
+	int bufix = 0, six = 0, tix = 0;
 	int i;
 	int ix;
 	int tto;
+	int bufend, sbufend;
 	struct data_string od;
 	struct option_cache *oc;
 	unsigned code;
-	int optstart;
+
+	if (first_cutoff) {
+	    if (first_cutoff >= buflen)
+		log_fatal("%s:%d:store_options: Invalid first cutoff.", MDL);
+
+	    bufend = first_cutoff;
+	} else
+	    bufend = buflen;
+
+	if (second_cutoff) {
+	    if (second_cutoff >= buflen)
+		log_fatal("%s:%d:store_options: Invalid second cutoff.", MDL);
+
+	    sbufend = second_cutoff;
+	} else
+	    sbufend = buflen;
 
 	memset (&od, 0, sizeof od);
 
@@ -774,10 +787,11 @@ int store_options (buffer, buflen, packet, lease, client_state,
 	    /* Number of bytes left to store (some may already
 	       have been stored by a previous pass). */
 	    unsigned length;
-	    int optstart;
+	    int optstart, soptstart, toptstart;
 	    struct universe *u;
 	    int have_encapsulation = 0;
 	    struct data_string encapsulation;
+	    int splitup;
 
 	    memset (&encapsulation, 0, sizeof encapsulation);
 
@@ -914,50 +928,285 @@ int store_options (buffer, buflen, packet, lease, client_state,
 	       in any case, if the option data will cross a buffer
 	       boundary, split it across that boundary. */
 
+
+	    if (length > 255)
+		splitup = 1;
+	    else
+		splitup = 0;
+
 	    ix = 0;
 	    optstart = bufix;
+	    soptstart = six;
+	    toptstart = tix;
 	    while (length) {
-		    unsigned char incr = length > 255 ? 255 : length;
+		    unsigned incr = length;
 		    int consumed = 0;
-		    
-		    /* If this hunk of the buffer will cross a
-		       boundary, only go up to the boundary in this
-		       pass. */
-		    if (bufix < first_cutoff &&
-			bufix + incr > first_cutoff)
-			    incr = first_cutoff - bufix;
-		    else if (bufix < second_cutoff &&
-			     bufix + incr > second_cutoff)
-			    incr = second_cutoff - bufix;
-		    
-		    /* If this option is going to overflow the buffer,
-		       skip it. */
-		    if (bufix + 2 + incr > buflen) {
-			    bufix = optstart;
-			    break;
-		    }
-		    
-		    /* Everything looks good - copy it in! */
-		    buffer [bufix] = code;
-		    buffer [bufix + 1] = incr;
-		    if (tto && incr == length) {
-			    memcpy (buffer + bufix + 2,
-				    od.data + ix, (unsigned)(incr - 1));
-			    buffer [bufix + 2 + incr - 1] = 0;
+		    int *pix;
+		    unsigned char *base;
+
+		    /* Try to fit it in the options buffer. */
+		    if (!splitup &&
+			((!six && !tix && (i == priority_len - 1) &&
+			  (bufix + 2 + length < bufend)) ||
+			 (bufix + 5 + length < bufend))) {
+			base = buffer;
+			pix = &bufix;
+		    /* Try to fit it in the second buffer. */
+		    } else if (!splitup && first_cutoff &&
+			       (first_cutoff + six + 3 + length < sbufend)) {
+			base = &buffer[first_cutoff];
+			pix = &six;
+		    /* Try to fit it in the third buffer. */
+		    } else if (!splitup && second_cutoff &&
+			       (second_cutoff + tix + 3 + length < buflen)) {
+			base = &buffer[second_cutoff];
+			pix = &tix;
+		    /* Split the option up into the remaining space. */
 		    } else {
-			    memcpy (buffer + bufix + 2,
+			splitup = 1;
+
+			/* Use any remaining options space. */
+			if (bufix + 6 < bufend) {
+			    incr = bufend - bufix - 5;
+			    base = buffer;
+			    pix = &bufix;
+			/* Use any remaining first_cutoff space. */
+			} else if (first_cutoff &&
+				   (first_cutoff + six + 4 < sbufend)) {
+			    incr = sbufend - (first_cutoff + six) - 3;
+			    base = &buffer[first_cutoff];
+			    pix = &six;
+			/* Use any remaining second_cutoff space. */
+			} else if (second_cutoff &&
+				   (second_cutoff + tix + 4 < buflen)) {
+			    incr = buflen - (second_cutoff + tix) - 3;
+			    base = &buffer[second_cutoff];
+			    pix = &tix;
+			/* Give up, roll back this option. */
+			} else {
+			    bufix = optstart;
+			    six = soptstart;
+			    tix = toptstart;
+			    break;
+			}
+		    }
+
+		    if (incr > length)
+			incr = length;
+		    if (incr > 255)
+			incr = 255;
+
+		    /* Everything looks good - copy it in! */
+		    base [*pix] = code;
+		    base [*pix + 1] = (unsigned char)incr;
+		    if (tto && incr == length) {
+			    if (incr > 1)
+				memcpy (base + *pix + 2,
+					od.data + ix, (unsigned)(incr - 1));
+			    base [*pix + 2 + incr - 1] = 0;
+		    } else {
+			    memcpy (base + *pix + 2,
 				    od.data + ix, (unsigned)incr);
 		    }
 		    length -= incr;
 		    ix += incr;
-		    bufix += 2 + incr;
+		    *pix += 2 + incr;
 	    }
 	    data_string_forget (&od, MDL);
 	}
 
+	/* If we can overload, and we have, then PAD and END those spaces. */
+	if (first_cutoff && six) {
+	    if ((first_cutoff + six + 1) < sbufend)
+		memset (&buffer[first_cutoff + six + 1], DHO_PAD,
+			sbufend - (first_cutoff + six + 1));
+	    else if (first_cutoff + six >= sbufend)
+		log_fatal("Second buffer overflow in overloaded options.");
+
+	    buffer[first_cutoff + six] = DHO_END;
+	    *ocount |= 1; /* So that caller knows there's data there. */
+	}
+
+	if (second_cutoff && tix) {
+	    if (second_cutoff + tix + 1 < buflen) {
+		memset (&buffer[second_cutoff + tix + 1], DHO_PAD,
+			buflen - (second_cutoff + tix + 1));
+	    } else if (second_cutoff + tix >= buflen)
+		log_fatal("Third buffer overflow in overloaded options.");
+
+	    buffer[second_cutoff + tix] = DHO_END;
+	    *ocount |= 2; /* So that caller knows there's data there. */
+	}
+
+	if ((six || tix) && (bufix + 3 > bufend))
+	    log_fatal("Not enough space for option overload option.");
+
 	return bufix;
 }
 
+/* Return true if the format string has a variable length text option
+ * ("t"), return false otherwise.
+ */
+
+int
+format_has_text(format)
+	const char *format;
+{
+	const char *p;
+	int retval = 0;
+
+	p = format;
+	while (*p != '\0') {
+		switch (*p++) {
+		    case 'd':
+		    case 't':
+			return 1;
+
+			/* These symbols are arbitrary, not fixed or
+			 * determinable length...text options with them is
+			 * invalid.
+			 */
+		    case 'A':
+		    case 'a':
+		    case 'X':
+		    case 'x':
+			return 0;
+
+			/* 'E' is variable length, but not arbitrary...you
+			 * can find its length if you can find an END option.
+			 * N is one-byte in length but trails a name of a
+			 * space defining the enumeration values.  So treat
+			 * both the same - valid, fixed-length fields.
+			 */
+		    case 'E':
+		    case 'N':
+			/* Consume the space name. */
+			while ((*p != '\0') && (*p++ != '.'))
+				;
+			break;
+
+		    default:
+			break;
+		}
+	}
+
+	return 0;
+}
+
+/* Determine the minimum length of a DHCP option prior to any variable
+ * or inconsistent length formats, according to its configured format
+ * variable (and possibly from supplied option cache contents for variable
+ * length format symbols).
+ */
+
+int
+format_min_length(format, oc)
+	const char *format;
+	struct option_cache *oc;
+{
+	const char *p;
+	int min_len = 0;
+	int last_size = 0;
+
+	p = format;
+	while (*p != '\0') {
+		switch (*p++) {
+		    case 'I': /* IPv4 Address */
+		    case 'l': /* int32_t */
+		    case 'L': /* uint32_t */
+		    case 'T': /* Lease Time, uint32_t equivalent */
+			min_len += 4;
+			last_size = 4;
+			break;
+
+		    case 's': /* int16_t */
+		    case 'S': /* uint16_t */
+			min_len += 2;
+			last_size = 2;
+			break;
+
+		    case 'N': /* Enumeration in 1-byte values. */
+			/* Consume space name. */
+			while ((*p != '\0') && (*p++ != '.'))
+				;
+
+			/* Fall Through to handle as one-byte field */
+
+		    case 'b': /* int8_t */
+		    case 'B': /* uint8_t */
+		    case 'F': /* Flag that is always true. */
+		    case 'f': /* Flag */
+			min_len++;
+			last_size = 1;
+			break;
+
+		    case 'o': /* Last argument is optional. */
+			min_len -= last_size;
+		    case 'e': /* Encapsulation hint (there is an 'E' later). */
+			last_size = 0;
+			break;
+
+		    case 'E': /* Encapsulated options. */
+			/* Consume space name. */
+			while ((*p != '\0') && (*p++ != '.'))
+				;
+
+			/* Find an end option, or find that the encaps options
+			 * go all the way to the end (or beyond) of the data
+			 * portion of the option.
+			 */
+			last_size = 0;
+			while (min_len < oc->data.len) {
+				if (oc->data.data[min_len] == DHO_END) {
+					min_len++;
+					last_size++;
+					break;
+				} else if (oc->data.data[min_len] == DHO_PAD) {
+					min_len++;
+					last_size++;
+				} else if ((min_len + 1) < oc->data.len) {
+					min_len += oc->data.data[min_len+1]+2;
+					last_size += oc->data.data[min_len+1]+2;
+				} else {
+					/* Suboption length is out of bounds,
+					 * advance beyond the code/length pair
+					 * to trigger below error conditonal.
+					 */
+					min_len += 2;
+					last_size += 2;
+					break;
+				}
+			}
+
+			if (min_len > oc->data.len) {
+				log_error("format_min_length(%s): "
+					  "Encapsulated options exceed "
+					  "supplied buffer.", format);
+				return INT_MAX;
+			}
+
+			break;
+
+		    case 'd': /* "Domain name" */
+		    case 't': /* "ASCII Text" */
+		    case 'X': /* "ASCII or Hex Conditional */
+		    case 'x': /* "Hex" */
+		    case 'A': /* Array of all that precedes. */
+		    case 'a': /* Array of preceding symbol. */
+			return min_len;
+
+		    default:
+			/* No safe value is known. */
+			log_error("format_min_length(%s): No safe value "
+				  "for unknown format symbols.", format);
+			return INT_MAX;
+		}
+	}
+
+	return min_len;
+}
+
+
 /* Format the specified option so that a human can easily read it. */
 
 const char *pretty_print_option (option, data, len, emit_commas, emit_quotes)
@@ -1412,6 +1661,21 @@ int save_option_buffer (struct universe *universe,
 	
 	op -> option = option;
 
+	/* If this option is ultimately a text option, null determinate to
+	 * comply with RFC2132 section 2.  Mark a flag so this can be sensed
+	 * later to echo NULLs back to clients that supplied them (they
+	 * probably expect them).
+	 */
+	if (format_has_text(option->format)) {
+		int min_len = format_min_length(option->format, op);
+
+		while ((op->data.len > min_len) &&
+		       (op->data.data[op->data.len-1] == '\0')) {
+			op->data.len--;
+			op->flags |= OPTION_HAD_NULLS;
+		}
+	}
+
 	/* Now store the option. */
 	save_option (universe, options, op);
 
